@@ -127,140 +127,180 @@ int SLgetkey_map_to_ansi (int enable)
    return 0;
 }
 
-unsigned int _SLpc_convert_scancode (unsigned int scan)
+static int convert_scancode (unsigned int scan, 
+			     unsigned int shift,
+			     int getkey,
+			     unsigned int *ret_key)
 {
-   char *keystr;
+   unsigned char buf[16];
+   unsigned char *b;
+   unsigned char end;
+   int is_arrow;
 
-   if (Map_To_ANSI == 0)
+   shift &= (_SLTT_KEY_ALT|_SLTT_KEY_SHIFT|_SLTT_KEY_CTRL);
+
+   b = buf;
+   if (_SLTT_KEY_ALT == shift)
      {
-	SLang_ungetkey (scan);
-	return 0;
+	shift = 0;
+	*b++ = 27;
+     }
+   *b++ = 27;
+   *b++ = '[';
+
+   is_arrow = 0;
+   end = '~';
+   if (shift)
+     {
+	if (shift == _SLTT_KEY_CTRL)
+	  end = '^';
+	else if (shift == _SLTT_KEY_SHIFT)
+	  end = '$';
+	else shift = 0;
      }
 
    /* These mappings correspond to what rxvt produces under Linux */
-   switch (scan)
+   switch (scan & 0xFF)
      {
       default:
-	return 0;
+	return -1;
 
-      case 'G':			       /* home */
-	keystr = "[1~";
+      case 0x47:		       /* home */
+	*b++ = '1';
 	break;
-      case 'H':			       /* up */
-	keystr = "[A";
+      case 0x48:		       /* up */
+	end = 'A';
+	is_arrow = 1;
 	break;
-      case 'I':			       /* PgUp */
-	keystr = "[5~";
+      case 0x49:		       /* PgUp */
+	*b++ = '5';
 	break;
-      case 'K':			       /* Left */
-	keystr = "[D";
+      case 0x4B:		       /* Left */
+	end = 'D';
+	is_arrow = 1;
 	break;
-      case 'M':			       /* Right */
-	keystr = "[C";
+      case 0x4D:		       /* Right */
+	end = 'C';
+	is_arrow = 1;
 	break;
-      case 'O':			       /* End */
-	keystr = "[4~";
+      case 0x4F:		       /* End */
+	*b++ = '4';
 	break;
-      case 'P':			       /* Down */
-	keystr = "[B";
+      case 0x50:		       /* Down */
+	end = 'B';
+	is_arrow = 1;
 	break;
-      case 'Q':			       /* PgDn */
-	keystr = "[6~";
+      case 0x51:		       /* PgDn */
+	*b++ = '6';
 	break;
-      case 'R':			       /* Insert */
-	keystr = "[2~";
+      case 0x52:		       /* Insert */
+	*b++ = '2';
 	break;
-      case 'S':			       /* Delete */
-	keystr = "[3~";
+      case 0x53:		       /* Delete */
+	*b++ = '3';
 	break;
-
       case ';':			       /* F1 */
-      case 'T':			       /* (shift) */
-      case '^':			       /* (ctrl) */
-      case 'h':			       /* (alt) */
-	keystr = "[11~";
+	*b++ = '1';
+	*b++ = '1';
 	break;
-
       case '<':			       /* F2 */
-      case 'U':			       /* (shift) */
-      case '_':			       /* (ctrl) */
-      case 'i':			       /* (alt) */
-	keystr = "[12~";
+	*b++ = '1';
+	*b++ = '2';
 	break;
-
       case '=':			       /* F3 */
-      case 'V':			       /* (shift) */
-      case '`':			       /* (ctrl) */
-      case 'j':			       /* (alt) */
-	keystr = "[13~";
+	*b++ = '1';
+	*b++ = '3';
 	break;
 
       case '>':			       /* F4 */
-      case 'W':			       /* (shift) */
-      case 'a':			       /* (ctrl) */
-      case 'k':			       /* (alt) */
-	keystr = "[14~";
+	*b++ = '1';
+	*b++ = '4';
 	break;
 
       case '?':			       /* F5 */
-      case 'X':			       /* (shift) */
-      case 'b':			       /* (ctrl) */
-      case 'l':			       /* (alt) */
-	keystr = "[15~";
+	*b++ = '1';
+	*b++ = '5';
 	break;
 
       case '@':			       /* F6 */
-      case 'Y':			       /* (shift) */
-      case 'c':			       /* (ctrl) */
-      case 'm':			       /* (alt) */
-	keystr = "[17~";
+	*b++ = '1';
+	*b++ = '7';
 	break;
 
       case 'A':			       /* F7 */
-      case 'Z':			       /* (shift) */
-      case 'd':			       /* (ctrl) */
-      case 'n':			       /* (alt) */
-	keystr = "[18~";
+	*b++ = '1';
+	*b++ = '8';
 	break;
 
       case 'B':			       /* F8 */
-      case '[':			       /* (shift) */
-      case 'e':			       /* (ctrl) */
-      case 'o':			       /* (alt) */
-	keystr = "[19~";
+	*b++ = '1';
+	*b++ = '9';
 	break;
 
       case 'C':			       /* F9 */
-      case '\\':		       /* (shift) */
-      case 'f':			       /* (ctrl) */
-      case 'p':			       /* (alt) */
-	keystr = "[20~";
+	*b++ = '2';
+	*b++ = '0';
 	break;
 
       case 'D':			       /* F10 */
-      case ']':			       /* (shift) */
-      case 'g':			       /* (ctrl) */
-      case 'q':			       /* (alt) */
-	keystr = "[21~";
+	*b++ = '2';
+	*b++ = '1';
 	break;
 
-      case 133:			       /* F11 */
-      case 135:			       /* (shift) */
-      case 137:			       /* (ctrl) */
-      case 139:			       /* (alt) */
-	keystr = "[23~";
+      case 0x57:		       /* F11 */
+	*b++ = '2';
+	*b++ = '3';
 	break;
 
-      case 134:			       /* F12 */
-      case 136:			       /* (shift) */
-      case 138:			       /* (ctrl) */
-      case 140:			       /* (alt) */
-	keystr = "[24~";
+      case 0x58:		       /* F12 */
+	*b++ = '2';
+	*b++ = '4';
 	break;
      }
 
-   (void) SLang_ungetkey_string ((unsigned char *)keystr, strlen (keystr));
-   return 27;
+   if (is_arrow && shift)
+     {
+	if (shift == _SLTT_KEY_CTRL)
+	  end &= 0x1F;
+	else
+	  end |= 0x20;
+     }
+   *b++ = end;
+   
+   if (getkey)
+     {
+	(void) SLang_buffer_keystring (buf + 1, (unsigned int) (b - (buf + 1)));
+	*ret_key = buf[0];
+	return 0;
+     }
+
+   (void) SLang_buffer_keystring (buf, (unsigned int) (b - buf));
+   return 0;
+}
+
+   
+unsigned int _SLpc_convert_scancode (unsigned int scan,
+				     unsigned int shift,
+				     int getkey)
+{
+   unsigned char buf[16];
+
+   if (Map_To_ANSI)
+     {
+	if (0 == convert_scancode (scan, shift, getkey, &scan))
+	  return scan;
+     }
+   
+   if (getkey)
+     {
+	buf[0] = scan & 0xFF;
+	SLang_buffer_keystring (buf, 1);
+	return (scan >> 8) & 0xFF;
+     }
+   buf[0] = (scan >> 8) & 0xFF;
+   buf[1] = scan & 0xFF;
+   (void) SLang_buffer_keystring (buf, 2);
+   return 0;
 }
 
 #endif
