@@ -915,13 +915,21 @@ void _SLparse_start (SLang_Load_Type *llt)
    llt->parse_level = 0;
    statement_list (&ctok);
 
+   if ((SLang_Error == 0)
+       && (ctok.type != EOF_TOKEN))
+     _SLparse_error ("Parse ended prematurely", &ctok, 0);
+   
+
    if (SLang_Error)
      {
 	if (SLang_Error < 0)	       /* severe error */
 	  save_list = NULL;
 
 	while (Token_List != save_list)
-	  pop_token_list (1);
+	  {
+	     if (-1 == pop_token_list (1))
+	       break;		       /* ??? when would this happen? */
+	  }
      }
 
    free_token (&ctok);
